@@ -10,7 +10,10 @@ fn main() {
     };
 
     let data = std::fs::read(target).unwrap();
-    let token = biscuit_auth::token::Biscuit::from(&data[..]).unwrap();
+    let token = biscuit_auth::UnverifiedBiscuit::from(&data[..]).unwrap();
 
-    println!("Token content:\n{}", token.print());
+    println!("Token content:");
+    for i in 0..token.block_count() {
+        println!("block {}:\n{}\n", i, token.print_block_source(i).unwrap());
+    }
 }
