@@ -97,6 +97,10 @@ fn main() {
 
     results.push(parsing(&mut rng, &target, &root, test));
 
+    results.push(default_symbols(&mut rng, &target, &root, test));
+
+    results.push(execution_scope(&mut rng, &target, &root, test));
+
     if json {
         let s = serde_json::to_string_pretty(&TestCases {
             root_private_key: hex::encode(root.private().to_bytes()),
@@ -147,6 +151,7 @@ struct Validation {
     world: Option<AuthorizerWorld>,
     result: AuthorizerResult,
     authorizer_code: String,
+    revocation_ids: Vec<String>,
 }
 
 impl TestResult {
@@ -181,7 +186,12 @@ impl TestResult {
                     "authorizer code:\n```\n{}```\n",
                     validation.authorizer_code
                 );
-                writeln!(&mut s, "authorizer world:\n```\nWorld {{\n  facts: {:#?}\n  rules: {:#?}\n  checks: {:#?}\n  policies: {:#?}\n}}\n```\n",
+
+                writeln!(&mut s, "revocation ids:");
+                for id in &validation.revocation_ids {
+                    writeln!(&mut s, "- `{}`", id);
+                }
+                writeln!(&mut s, "\nauthorizer world:\n```\nWorld {{\n  facts: {:#?}\n  rules: {:#?}\n  checks: {:#?}\n  policies: {:#?}\n}}\n```\n",
                          world.facts, world.rules, world.checks, world.policies);
             }
 
@@ -220,10 +230,17 @@ fn validate_token(
                 world: None,
                 authorizer_code: String::new(),
                 result: AuthorizerResult::Err(e),
+                revocation_ids: vec![],
             }
         }
     };
 
+    let mut revocation_ids = vec![];
+
+    for bytes in token.revocation_identifiers() {
+        revocation_ids.push(hex::encode(&bytes));
+    }
+
     let mut authorizer = match token.authorizer() {
         Ok(v) => v,
         Err(e) => {
@@ -231,6 +248,7 @@ fn validate_token(
                 world: None,
                 authorizer_code: String::new(),
                 result: AuthorizerResult::Err(e),
+                revocation_ids,
             }
         }
     };
@@ -238,7 +256,7 @@ fn validate_token(
     let mut authorizer_code = String::new();
     for fact in ambient_facts {
         authorizer_code += &format!("{};\n", fact);
-        authorizer.add_fact(fact);
+        authorizer.add_fact(fact).unwrap();
     }
 
     if !ambient_rules.is_empty() {
@@ -247,7 +265,7 @@ fn validate_token(
 
     for rule in ambient_rules {
         authorizer_code += &format!("{};\n", rule);
-        authorizer.add_rule(rule);
+        authorizer.add_rule(rule).unwrap();
     }
 
     if !checks.is_empty() {
@@ -255,7 +273,7 @@ fn validate_token(
     }
 
     for check in checks {
-        authorizer.add_check(&check[..]);
+        authorizer.add_check(&check[..]).unwrap();
         let c: Check = (&check[..]).try_into().unwrap();
         authorizer_code += &format!("{};\n", c);
     }
@@ -278,6 +296,7 @@ fn validate_token(
             Err(e) => AuthorizerResult::Err(e),
         },
         authorizer_code,
+        revocation_ids,
     }
 }
 
@@ -311,23 +330,31 @@ fn basic_token<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file2"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file1"), s("write")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("write")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
@@ -381,21 +408,25 @@ fn different_root_key<T: Rng + CryptoRng>(
     let root2 = KeyPair::new_with_rng(rng);
     let mut builder = Biscuit::builder(&root2);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
@@ -422,6 +453,7 @@ fn different_root_key<T: Rng + CryptoRng>(
             vec![],
         ),
     );
+
     TestResult {
         title,
         filename,
@@ -442,23 +474,31 @@ fn invalid_signature_format<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file2"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file1"), s("write")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("write")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
@@ -509,23 +549,31 @@ fn random_block<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file2"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file1"), s("write")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("write")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
@@ -579,23 +627,31 @@ fn invalid_signature<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file2"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file1"), s("write")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("write")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
@@ -648,34 +704,44 @@ fn reordered_blocks<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file2"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file1"), s("write")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("write")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
 
     let mut block3 = biscuit2.create_block();
 
-    block3.add_check(rule(
-        "check2",
-        &[var("0")],
-        &[pred("resource", &[string("file1")])],
-    ));
+    block3
+        .add_check(rule(
+            "check2",
+            &[var("0")],
+            &[pred("resource", &[string("file1")])],
+        ))
+        .unwrap();
 
     let keypair3 = KeyPair::new_with_rng(rng);
     let biscuit3 = biscuit2.append_with_keypair(&keypair3, block3).unwrap();
@@ -728,38 +794,48 @@ fn scoped_rules<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("user_id", &[string("alice")]));
-    builder.add_authority_fact(fact("owner", &[string("alice"), string("file1")]));
+    builder
+        .add_authority_fact(fact("user_id", &[string("alice")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("owner", &[string("alice"), string("file1")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_rule(rule(
-        "right",
-        &[var("0"), s("read")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("user_id", &[var("1")]),
-            pred("owner", &[var("1"), var("0")]),
-        ],
-    ));
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_rule(rule(
+            "right",
+            &[var("0"), s("read")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("user_id", &[var("1")]),
+                pred("owner", &[var("1"), var("0")]),
+            ],
+        ))
+        .unwrap();
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
 
     let mut block3 = biscuit2.create_block();
 
-    block3.add_fact(fact("owner", &[string("alice"), string("file2")]));
+    block3
+        .add_fact(fact("owner", &[string("alice"), string("file2")]))
+        .unwrap();
 
     let keypair3 = KeyPair::new_with_rng(rng);
     let biscuit3 = biscuit2.append_with_keypair(&keypair3, block3).unwrap();
@@ -812,28 +888,34 @@ fn scoped_checks<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
 
     let mut block3 = biscuit2.create_block();
 
-    block3.add_fact(fact("right", &[string("file2"), s("read")]));
+    block3
+        .add_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
 
     let keypair3 = KeyPair::new_with_rng(rng);
     let biscuit3 = biscuit2.append_with_keypair(&keypair3, block3).unwrap();
@@ -889,11 +971,13 @@ fn expired_token<T: Rng + CryptoRng>(
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[string("file1")],
-        &[pred("resource", &[string("file1")])],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[string("file1")],
+            &[pred("resource", &[string("file1")])],
+        ))
+        .unwrap();
     // January 1 2019
     block2.expiration_date(
         UNIX_EPOCH
@@ -960,13 +1044,17 @@ fn authorizer_scope<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_fact(fact("right", &[string("file2"), s("read")]));
+    block2
+        .add_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
@@ -1026,7 +1114,9 @@ fn authorizer_authority_checks<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
     token = print_blocks(&biscuit1);
@@ -1085,11 +1175,13 @@ fn authority_checks<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_check(rule(
-        "check1",
-        &[string("file1")],
-        &[pred("resource", &[string("file1")])],
-    ));
+    builder
+        .add_authority_check(rule(
+            "check1",
+            &[string("file1")],
+            &[pred("resource", &[string("file1")])],
+        ))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
     token = print_blocks(&biscuit1);
@@ -1153,8 +1245,12 @@ fn block_rules<T: Rng + CryptoRng>(
     let token;
 
     let mut builder = Biscuit::builder(&root);
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file2"), s("read")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
@@ -1164,21 +1260,23 @@ fn block_rules<T: Rng + CryptoRng>(
     let date1 = SystemTime::UNIX_EPOCH + Duration::from_secs(1924952399);
 
     // generate valid_date("file1") if before date1
-    block2.add_rule(constrained_rule(
-        "valid_date",
-        &[string("file1")],
-        &[
-            pred("time", &[variable("0")]),
-            pred("resource", &[string("file1")]),
-        ],
-        &[Expression {
-            ops: vec![
-                Op::Value(var("0")),
-                Op::Value(date(&date1)),
-                Op::Binary(Binary::LessOrEqual),
+    block2
+        .add_rule(constrained_rule(
+            "valid_date",
+            &[string("file1")],
+            &[
+                pred("time", &[variable("0")]),
+                pred("resource", &[string("file1")]),
             ],
-        }],
-    ));
+            &[Expression {
+                ops: vec![
+                    Op::Value(var("0")),
+                    Op::Value(date(&date1)),
+                    Op::Binary(Binary::LessOrEqual),
+                ],
+            }],
+        ))
+        .unwrap();
 
     // timestamp for Friday, December 31, 1999 12:59:59 PM UTC
     let date2 = SystemTime::UNIX_EPOCH + Duration::from_secs(946645199);
@@ -1187,40 +1285,44 @@ fn block_rules<T: Rng + CryptoRng>(
     strings.insert(string("file1"));
 
     // generate a valid date fact for any file other than "file1" if before date2
-    block2.add_rule(constrained_rule(
-        "valid_date",
-        &[variable("1")],
-        &[
-            pred("time", &[variable("0")]),
-            pred("resource", &[variable("1")]),
-        ],
-        &[
-            Expression {
-                ops: vec![
-                    Op::Value(var("0")),
-                    Op::Value(date(&date2)),
-                    Op::Binary(Binary::LessOrEqual),
-                ],
-            },
-            Expression {
-                ops: vec![
-                    Op::Value(set(strings)),
-                    Op::Value(var("1")),
-                    Op::Binary(Binary::Contains),
-                    Op::Unary(Unary::Negate),
-                ],
-            },
-        ],
-    ));
+    block2
+        .add_rule(constrained_rule(
+            "valid_date",
+            &[variable("1")],
+            &[
+                pred("time", &[variable("0")]),
+                pred("resource", &[variable("1")]),
+            ],
+            &[
+                Expression {
+                    ops: vec![
+                        Op::Value(var("0")),
+                        Op::Value(date(&date2)),
+                        Op::Binary(Binary::LessOrEqual),
+                    ],
+                },
+                Expression {
+                    ops: vec![
+                        Op::Value(set(strings)),
+                        Op::Value(var("1")),
+                        Op::Binary(Binary::Contains),
+                        Op::Unary(Unary::Negate),
+                    ],
+                },
+            ],
+        ))
+        .unwrap();
 
-    block2.add_check(rule(
-        "check1",
-        &[variable("0")],
-        &[
-            pred("valid_date", &[variable("0")]),
-            pred("resource", &[var("0")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[variable("0")],
+            &[
+                pred("valid_date", &[variable("0")]),
+                pred("resource", &[var("0")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
@@ -1301,18 +1403,20 @@ fn regex_constraint<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_check(constrained_rule(
-        "resource_match",
-        &[variable("0")],
-        &[pred("resource", &[variable("0")])],
-        &[Expression {
-            ops: vec![
-                Op::Value(var("0")),
-                Op::Value(string("file[0-9]+.txt")),
-                Op::Binary(Binary::Regex),
-            ],
-        }],
-    ));
+    builder
+        .add_authority_check(constrained_rule(
+            "resource_match",
+            &[variable("0")],
+            &[pred("resource", &[variable("0")])],
+            &[Expression {
+                ops: vec![
+                    Op::Value(var("0")),
+                    Op::Value(string("file[0-9]+.txt")),
+                    Op::Binary(Binary::Regex),
+                ],
+            }],
+        ))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
     token = print_blocks(&biscuit1);
@@ -1370,11 +1474,13 @@ fn multi_queries_checks<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact(
-        "must_be_present",
-        &[string("hello")],
-        //&[string("hello")],
-    ));
+    builder
+        .add_authority_fact(fact(
+            "must_be_present",
+            &[string("hello")],
+            //&[string("hello")],
+        ))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
     token = print_blocks(&biscuit1);
@@ -1433,11 +1539,13 @@ fn check_head_name<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_check(rule(
-        "check1",
-        &[s("test")],
-        &[pred("resource", &[s("hello")])],
-    ));
+    builder
+        .add_authority_check(rule(
+            "check1",
+            &[s("test")],
+            &[pred("resource", &[s("hello")])],
+        ))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
@@ -1487,62 +1595,106 @@ fn expressions<T: Rng + CryptoRng>(
     let mut builder = Biscuit::builder(&root);
 
     //boolean true
-    builder.add_authority_check("check if true");
+    builder.add_authority_check("check if true").unwrap();
     //boolean false and negation
-    builder.add_authority_check("check if !false");
+    builder.add_authority_check("check if !false").unwrap();
     //boolean and
-    builder.add_authority_check("check if !false and true");
+    builder
+        .add_authority_check("check if !false && true")
+        .unwrap();
     //boolean or
-    builder.add_authority_check("check if false or true");
+    builder
+        .add_authority_check("check if false or true")
+        .unwrap();
     //boolean parens
-    builder.add_authority_check("check if (true or false) and true");
+    builder
+        .add_authority_check("check if (true || false) && true")
+        .unwrap();
 
     //integer less than
-    builder.add_authority_check("check if 1 < 2");
+    builder.add_authority_check("check if 1 < 2").unwrap();
     //integer greater than
-    builder.add_authority_check("check if 2 > 1");
+    builder.add_authority_check("check if 2 > 1").unwrap();
     //integer less or equal
-    builder.add_authority_check("check if 1 <= 2");
-    builder.add_authority_check("check if 1 <= 1");
+    builder.add_authority_check("check if 1 <= 2").unwrap();
+    builder.add_authority_check("check if 1 <= 1").unwrap();
     //integer greater or equal
-    builder.add_authority_check("check if 2 >= 1");
-    builder.add_authority_check("check if 2 >= 2");
+    builder.add_authority_check("check if 2 >= 1").unwrap();
+    builder.add_authority_check("check if 2 >= 2").unwrap();
     //integer equal
-    builder.add_authority_check("check if 3 == 3");
+    builder.add_authority_check("check if 3 == 3").unwrap();
     //integer add sub mul div
-    builder.add_authority_check("check if 1 + 2 * 3 - 4 /2 == 5");
+    builder
+        .add_authority_check("check if 1 + 2 * 3 - 4 /2 == 5")
+        .unwrap();
 
     // string prefix and suffix
     builder.add_authority_check(
         "check if \"hello world\".starts_with(\"hello\") && \"hello world\".ends_with(\"world\")",
-    );
+    ).unwrap();
     // string regex
-    builder.add_authority_check("check if \"aaabde\".matches(\"a*c?.e\")");
+    builder
+        .add_authority_check("check if \"aaabde\".matches(\"a*c?.e\")")
+        .unwrap();
+    // string contains
+    builder
+        .add_authority_check("check if \"aaabde\".contains(\"abd\")")
+        .unwrap();
+    // string concatenation
+    builder
+        .add_authority_check("check if \"aaabde\" == \"aaa\" + \"b\" + \"de\"")
+        .unwrap();
     // string equal
-    builder.add_authority_check("check if \"abcD12\" == \"abcD12\"");
+    builder
+        .add_authority_check("check if \"abcD12\" == \"abcD12\"")
+        .unwrap();
 
     //date less than
-    builder.add_authority_check("check if 2019-12-04T09:46:41+00:00 < 2020-12-04T09:46:41+00:00");
+    builder
+        .add_authority_check("check if 2019-12-04T09:46:41+00:00 < 2020-12-04T09:46:41+00:00")
+        .unwrap();
     //date greater than
-    builder.add_authority_check("check if 2020-12-04T09:46:41+00:00 > 2019-12-04T09:46:41+00:00");
+    builder
+        .add_authority_check("check if 2020-12-04T09:46:41+00:00 > 2019-12-04T09:46:41+00:00")
+        .unwrap();
     //date less or equal
-    builder.add_authority_check("check if 2019-12-04T09:46:41+00:00 <= 2020-12-04T09:46:41+00:00");
-    builder.add_authority_check("check if 2020-12-04T09:46:41+00:00 >= 2020-12-04T09:46:41+00:00");
+    builder
+        .add_authority_check("check if 2019-12-04T09:46:41+00:00 <= 2020-12-04T09:46:41+00:00")
+        .unwrap();
+    builder
+        .add_authority_check("check if 2020-12-04T09:46:41+00:00 >= 2020-12-04T09:46:41+00:00")
+        .unwrap();
     //date greater or equal
-    builder.add_authority_check("check if 2020-12-04T09:46:41+00:00 >= 2019-12-04T09:46:41+00:00");
-    builder.add_authority_check("check if 2020-12-04T09:46:41+00:00 >= 2020-12-04T09:46:41+00:00");
+    builder
+        .add_authority_check("check if 2020-12-04T09:46:41+00:00 >= 2019-12-04T09:46:41+00:00")
+        .unwrap();
+    builder
+        .add_authority_check("check if 2020-12-04T09:46:41+00:00 >= 2020-12-04T09:46:41+00:00")
+        .unwrap();
     //date equal
-    builder.add_authority_check("check if 2020-12-04T09:46:41+00:00 == 2020-12-04T09:46:41+00:00");
+    builder
+        .add_authority_check("check if 2020-12-04T09:46:41+00:00 == 2020-12-04T09:46:41+00:00")
+        .unwrap();
 
     //bytes equal
-    builder.add_authority_check("check if hex:12ab == hex:12ab");
+    builder
+        .add_authority_check("check if hex:12ab == hex:12ab")
+        .unwrap();
 
     // set contains
-    builder.add_authority_check("check if [1, 2].contains(2)");
-    builder.add_authority_check("check if [2020-12-04T09:46:41+00:00, 2019-12-04T09:46:41+00:00].contains(2020-12-04T09:46:41+00:00)");
-    builder.add_authority_check("check if [true, false, true].contains(true)");
-    builder.add_authority_check("check if [\"abc\", \"def\"].contains(\"abc\")");
-    builder.add_authority_check("check if [hex:12ab, hex:34de].contains(hex:34de)");
+    builder
+        .add_authority_check("check if [1, 2].contains(2)")
+        .unwrap();
+    builder.add_authority_check("check if [2020-12-04T09:46:41+00:00, 2019-12-04T09:46:41+00:00].contains(2020-12-04T09:46:41+00:00)").unwrap();
+    builder
+        .add_authority_check("check if [true, false, true].contains(true)")
+        .unwrap();
+    builder
+        .add_authority_check("check if [\"abc\", \"def\"].contains(\"abc\")")
+        .unwrap();
+    builder
+        .add_authority_check("check if [hex:12ab, hex:34de].contains(hex:34de)")
+        .unwrap();
 
     let biscuit = builder.build_with_rng(rng).unwrap();
     token = print_blocks(&biscuit);
@@ -1583,11 +1735,13 @@ fn unbound_variables_in_rule<T: Rng + CryptoRng>(
     let token;
 
     let mut builder = Biscuit::builder(&root);
-    builder.add_authority_check(rule(
-        "check1",
-        &[s("test")],
-        &[pred("operation", &[s("read")])],
-    ));
+    builder
+        .add_authority_check(rule(
+            "check1",
+            &[s("test")],
+            &[pred("operation", &[s("read")])],
+        ))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
@@ -1647,11 +1801,13 @@ fn generating_ambient_from_variables<T: Rng + CryptoRng>(
     let token;
 
     let mut builder = Biscuit::builder(&root);
-    builder.add_authority_check(rule(
-        "check1",
-        &[s("test")],
-        &[pred("operation", &[s("read")])],
-    ));
+    builder
+        .add_authority_check(rule(
+            "check1",
+            &[s("test")],
+            &[pred("operation", &[s("read")])],
+        ))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
@@ -1715,23 +1871,31 @@ fn sealed_token<T: Rng + CryptoRng>(
 
     let mut builder = Biscuit::builder(&root);
 
-    builder.add_authority_fact(fact("right", &[string("file1"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file2"), s("read")]));
-    builder.add_authority_fact(fact("right", &[string("file1"), s("write")]));
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file2"), s("read")]))
+        .unwrap();
+    builder
+        .add_authority_fact(fact("right", &[string("file1"), s("write")]))
+        .unwrap();
 
     let biscuit1 = builder.build_with_rng(rng).unwrap();
 
     let mut block2 = biscuit1.create_block();
 
-    block2.add_check(rule(
-        "check1",
-        &[var("0")],
-        &[
-            pred("resource", &[var("0")]),
-            pred("operation", &[s("read")]),
-            pred("right", &[var("0"), s("read")]),
-        ],
-    ));
+    block2
+        .add_check(rule(
+            "check1",
+            &[var("0")],
+            &[
+                pred("resource", &[var("0")]),
+                pred("operation", &[s("read")]),
+                pred("right", &[var("0"), s("read")]),
+            ],
+        ))
+        .unwrap();
 
     let keypair2 = KeyPair::new_with_rng(rng);
     let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
@@ -1829,6 +1993,121 @@ fn parsing<T: Rng + CryptoRng>(
     }
 }
 
+fn default_symbols<T: Rng + CryptoRng>(
+    rng: &mut T,
+    target: &str,
+    root: &KeyPair,
+    test: bool,
+) -> TestResult {
+    let title = "default_symbols".to_string();
+    let filename = "test22_default_symbols.bc".to_string();
+    let token;
+
+    let mut builder = Biscuit::builder(&root);
+
+    builder
+        .add_code(
+            r#"read(0);write(1);resource(2);operation(3);right(4);time(5);
+            role(6);owner(7);tenant(8);namespace(9);user(10);team(11);
+            service(12);admin(13);email(14);group(15);member(16);
+            ip_address(17);client(18);client_ip(19);domain(20);path(21);
+            version(22);cluster(23);node(24);hostname(25);nonce(26);query(27)"#,
+        )
+        .unwrap();
+
+    let biscuit1 = builder.build_with_rng(rng).unwrap();
+    token = print_blocks(&biscuit1);
+
+    let data = if test {
+        let v = load_testcase(target, "test22_default_symbols");
+        let expected = Biscuit::from(&v[..], |_| root.public()).unwrap();
+        print_diff(&biscuit1.print(), &expected.print());
+        v
+    } else {
+        let data = biscuit1.to_vec().unwrap();
+        write_testcase(target, "test22_default_symbols", &data[..]);
+        data
+    };
+
+    let check: Check = r#"check if read(0),write(1),resource(2),operation(3),right(4),
+        time(5),role(6),owner(7),tenant(8),namespace(9),user(10),team(11),
+        service(12),admin(13),email(14),group(15),member(16),ip_address(17),
+        client(18),client_ip(19),domain(20),path(21),version(22),cluster(23),
+        node(24),hostname(25),nonce(26),query(27)"#
+        .parse()
+        .unwrap();
+    let mut validations = BTreeMap::new();
+    validations.insert(
+        "".to_string(),
+        validate_token(root, &data[..], vec![], vec![], vec![check.queries]),
+    );
+
+    TestResult {
+        title,
+        filename,
+        token,
+        validations,
+    }
+}
+
+fn execution_scope<T: Rng + CryptoRng>(
+    rng: &mut T,
+    target: &str,
+    root: &KeyPair,
+    test: bool,
+) -> TestResult {
+    let title = "execution scope".to_string();
+    let filename = "test23_execution_scope.bc".to_string();
+    let token;
+
+    let mut builder = Biscuit::builder(&root);
+
+    builder.add_authority_fact("authority_fact(1)").unwrap();
+
+    let biscuit1 = builder.build_with_rng(rng).unwrap();
+
+    let mut block2 = biscuit1.create_block();
+
+    block2.add_fact("block1_fact(1)").unwrap();
+
+    let keypair2 = KeyPair::new_with_rng(rng);
+    let biscuit2 = biscuit1.append_with_keypair(&keypair2, block2).unwrap();
+
+    let mut block3 = biscuit2.create_block();
+
+    block3.add_check("check if authority_fact($var)").unwrap();
+    block3.add_check("check if block1_fact($var)").unwrap();
+
+    let keypair3 = KeyPair::new_with_rng(rng);
+    let biscuit3 = biscuit2.append_with_keypair(&keypair3, block3).unwrap();
+    token = print_blocks(&biscuit3);
+
+    let data = if test {
+        let v = load_testcase(target, "test23_execution_scope");
+        let expected = Biscuit::from(&v[..], |_| root.public()).unwrap();
+        print_diff(&biscuit3.print(), &expected.print());
+        v
+    } else {
+        let data = biscuit3.to_vec().unwrap();
+        write_testcase(target, "test23_execution_scope", &data[..]);
+
+        data
+    };
+
+    let mut validations = BTreeMap::new();
+    validations.insert(
+        "".to_string(),
+        validate_token(root, &data[..], vec![], vec![], vec![]),
+    );
+
+    TestResult {
+        title,
+        filename,
+        token,
+        validations,
+    }
+}
+
 fn print_blocks(token: &Biscuit) -> Vec<BlockContent> {
     let mut v = Vec::new();
 
