@@ -1,24 +1,72 @@
 //! Symbol table implementation
+use std::collections::HashSet;
 use time::{format_description::well_known::Rfc3339, OffsetDateTime};
+
 pub type SymbolIndex = u64;
 use super::{Check, Fact, Predicate, Rule, Term, World};
 
 #[derive(Clone, Debug, PartialEq, Default)]
 pub struct SymbolTable {
-    pub symbols: Vec<String>,
+    symbols: Vec<String>,
 }
 
+const DEFAULT_SYMBOLS: [&str; 28] = [
+    "read",
+    "write",
+    "resource",
+    "operation",
+    "right",
+    "time",
+    "role",
+    "owner",
+    "tenant",
+    "namespace",
+    "user",
+    "team",
+    "service",
+    "admin",
+    "email",
+    "group",
+    "member",
+    "ip_address",
+    "client",
+    "client_ip",
+    "domain",
+    "path",
+    "version",
+    "cluster",
+    "node",
+    "hostname",
+    "nonce",
+    "query",
+];
+
+const OFFSET: usize = 1024;
+
 impl SymbolTable {
     pub fn new() -> Self {
-        SymbolTable::default()
+        SymbolTable { symbols: vec![] }
+    }
+
+    //FIXME: should check if symbols are already in default
+    pub fn from(symbols: Vec<String>) -> Self {
+        SymbolTable { symbols }
+    }
+
+    pub fn extend(&mut self, other: &SymbolTable) {
+        self.symbols.extend(other.symbols.iter().cloned());
     }
 
     pub fn insert(&mut self, s: &str) -> SymbolIndex {
+        if let Some(index) = DEFAULT_SYMBOLS.iter().position(|sym| *sym == s) {
+            return index as u64;
+        }
+
         match self.symbols.iter().position(|sym| sym.as_str() == s) {
-            Some(index) => index as u64,
+            Some(index) => (OFFSET + index) as u64,
             None => {
                 self.symbols.push(s.to_string());
-                (self.symbols.len() - 1) as u64
+                (OFFSET + (self.symbols.len() - 1)) as u64
             }
         }
     }
@@ -29,19 +77,47 @@ impl SymbolTable {
     }
 
     pub fn get(&self, s: &str) -> Option<SymbolIndex> {
+        if let Some(index) = DEFAULT_SYMBOLS.iter().position(|sym| *sym == s) {
+            return Some(index as u64);
+        }
+
         self.symbols
             .iter()
             .position(|sym| sym.as_str() == s)
-            .map(|i| i as SymbolIndex)
+            .map(|i| (OFFSET + i) as SymbolIndex)
+    }
+
+    pub fn strings(&self) -> Vec<String> {
+        self.symbols.clone()
+    }
+
+    pub fn current_offset(&self) -> usize {
+        self.symbols.len()
+    }
+
+    pub fn split_at(&mut self, offset: usize) -> SymbolTable {
+        let mut table = SymbolTable::new();
+        table.symbols = self.symbols.split_off(offset);
+        table
+    }
+
+    pub fn is_disjoint(&self, other: &SymbolTable) -> bool {
+        let h1 = self.symbols.iter().collect::<HashSet<_>>();
+        let h2 = other.symbols.iter().collect::<HashSet<_>>();
+
+        h1.is_disjoint(&h2)
     }
 
     pub fn get_symbol(&self, i: SymbolIndex) -> Option<&str> {
-        self.symbols.get(i as usize).map(|s| s.as_str())
+        if i >= 1024 {
+            self.symbols.get((i - 1024) as usize).map(|s| s.as_str())
+        } else {
+            DEFAULT_SYMBOLS.get(i as usize).map(|s| *s)
+        }
     }
 
     pub fn print_symbol(&self, i: SymbolIndex) -> String {
-        self.symbols
-            .get(i as usize)
+        self.get_symbol(i)
             .map(|s| s.to_string())
             .unwrap_or_else(|| format!("<{}?>", i))
     }
@@ -98,10 +174,7 @@ impl SymbolTable {
             .collect::<Vec<_>>();
         format!(
             "{}({})",
-            self.symbols
-                .get(p.name as usize)
-                .map(|s| s.as_str())
-                .unwrap_or("<?>"),
+            self.get_symbol(p.name).unwrap_or("<?>"),
             strings.join(", ")
         )
     }
@@ -147,3 +220,46 @@ impl SymbolTable {
         format!("check if {}", queries.join(" or "))
     }
 }
+
+#[derive(Clone, Debug, PartialEq)]
+pub struct TemporarySymbolTable<'a> {
+    base: &'a SymbolTable,
+    offset: usize,
+    symbols: Vec<String>,
+}
+
+impl<'a> TemporarySymbolTable<'a> {
+    pub fn new(base: &'a SymbolTable) -> Self {
+        let offset = OFFSET + base.current_offset();
+
+        TemporarySymbolTable {
+            base,
+            offset,
+            symbols: vec![],
+        }
+    }
+
+    pub fn get_symbol(&self, i: SymbolIndex) -> Option<&str> {
+        if i as usize >= self.offset {
+            self.symbols
+                .get(i as usize - self.offset)
+                .map(|s| s.as_str())
+        } else {
+            self.base.get_symbol(i)
+        }
+    }
+
+    pub fn insert(&mut self, s: &str) -> SymbolIndex {
+        if let Some(index) = self.base.get(s) {
+            return index as u64;
+        }
+
+        match self.symbols.iter().position(|sym| sym.as_str() == s) {
+            Some(index) => (self.offset + index) as u64,
+            None => {
+                self.symbols.push(s.to_string());
+                (self.offset + (self.symbols.len() - 1)) as u64
+            }
+        }
+    }
+}
