@@ -7,6 +7,7 @@ use crate::parser::parse_source;
 use crate::time::Instant;
 use prost::Message;
 use std::{
+    collections::HashSet,
     convert::{TryFrom, TryInto},
     default::Default,
     time::{Duration, SystemTime},
@@ -18,6 +19,7 @@ use std::{
 #[derive(Clone)]
 pub struct Authorizer<'t> {
     world: datalog::World,
+    block_worlds: Vec<datalog::World>,
     pub(crate) symbols: datalog::SymbolTable,
     checks: Vec<Check>,
     token_checks: Vec<Vec<datalog::Check>>,
@@ -48,6 +50,7 @@ impl<'t> Authorizer<'t> {
 
         Ok(Authorizer {
             world,
+            block_worlds: vec![],
             symbols,
             checks: vec![],
             token_checks: vec![],
@@ -82,6 +85,7 @@ impl<'t> Authorizer<'t> {
 
         Ok(Authorizer {
             world,
+            block_worlds: vec![],
             symbols,
             checks,
             token_checks: vec![],
@@ -102,15 +106,6 @@ impl<'t> Authorizer<'t> {
             self.world.facts.insert(fact);
         }
 
-        let mut revocation_ids = token.revocation_identifiers();
-        let revocation_id_sym = self.symbols.get("revocation_id").unwrap();
-        for (i, id) in revocation_ids.drain(..).enumerate() {
-            self.world.facts.insert(datalog::Fact::new(
-                revocation_id_sym,
-                &[datalog::Term::Integer(i as i64), datalog::Term::Bytes(id)],
-            ));
-        }
-
         for rule in token.authority.rules.iter().cloned() {
             let r = Rule::convert_from(&rule, &token.symbols);
             let rule = r.convert(&mut self.symbols);
@@ -229,8 +224,21 @@ impl<'t> Authorizer<'t> {
 
     /// run a query over the authorizer's Datalog engine to gather data
     ///
-    /// ```rust,compile_fail
+    /// this only sees facts from the authorizer and the authority block
+    /// ```rust
+    /// # use biscuit_auth::KeyPair;
+    /// # use biscuit_auth::Biscuit;
+    /// let keypair = KeyPair::new();
+    /// let mut builder = Biscuit::builder(&keypair);
+    /// builder.add_authority_fact("user(\"John Doe\", 42)");
+    ///
+    /// let biscuit = builder.build().unwrap();
+    ///
+    /// let mut authorizer = biscuit.authorizer().unwrap();
     /// let res: Vec<(String, i64)> = authorizer.query("data($name, $id) <- user($name, $id)").unwrap();
+    /// # assert_eq!(res.len(), 1);
+    /// # assert_eq!(res[0].0, "John Doe");
+    /// # assert_eq!(res[0].1, 42);
     /// ```
     pub fn query<R: TryInto<Rule>, T: TryFrom<Fact, Error = E>, E: Into<error::Token>>(
         &mut self,
@@ -244,6 +252,8 @@ impl<'t> Authorizer<'t> {
 
     /// run a query over the authorizer's Datalog engine to gather data
     ///
+    /// this only sees facts from the authorizer and the authority block
+    ///
     /// this method can specify custom runtime limits
     pub fn query_with_limits<R: TryInto<Rule>, T: TryFrom<Fact, Error = E>, E: Into<error::Token>>(
         &mut self,
@@ -268,6 +278,76 @@ impl<'t> Authorizer<'t> {
             .collect()
     }
 
+    /// run a query over the authorizer's Datalog engine to gather data
+    ///
+    /// this has access to the facts generated when evaluating all the blocks
+    ///
+    /// ```rust
+    /// # use biscuit_auth::KeyPair;
+    /// # use biscuit_auth::Biscuit;
+    /// let keypair = KeyPair::new();
+    /// let mut builder = Biscuit::builder(&keypair);
+    /// builder.add_authority_fact("user(\"John Doe\", 42)");
+    ///
+    /// let biscuit = builder.build().unwrap();
+    ///
+    /// let mut authorizer = biscuit.authorizer().unwrap();
+    /// let res: Vec<(String, i64)> = authorizer.query("data($name, $id) <- user($name, $id)").unwrap();
+    /// # assert_eq!(res.len(), 1);
+    /// # assert_eq!(res[0].0, "John Doe");
+    /// # assert_eq!(res[0].1, 42);
+    /// ```
+    pub fn query_all<R: TryInto<Rule>, T: TryFrom<Fact, Error = E>, E: Into<error::Token>>(
+        &mut self,
+        rule: R,
+    ) -> Result<Vec<T>, error::Token>
+    where
+        error::Token: From<<R as TryInto<Rule>>::Error>,
+    {
+        self.query_all_with_limits(rule, AuthorizerLimits::default())
+    }
+
+    /// run a query over the authorizer's Datalog engine to gather data
+    ///
+    /// this has access to the facts generated when evaluating all the blocks
+    ///
+    /// this method can specify custom runtime limits
+    pub fn query_all_with_limits<
+        R: TryInto<Rule>,
+        T: TryFrom<Fact, Error = E>,
+        E: Into<error::Token>,
+    >(
+        &mut self,
+        rule: R,
+        limits: AuthorizerLimits,
+    ) -> Result<Vec<T>, error::Token>
+    where
+        error::Token: From<<R as TryInto<Rule>>::Error>,
+    {
+        let rule = rule.try_into()?;
+
+        self.world
+            .run_with_limits(&self.symbols, limits.into())
+            .map_err(error::Token::RunLimit)?;
+        let rule = rule.convert(&mut self.symbols);
+        let mut res = self.world.query_rule(rule.clone(), &self.symbols);
+
+        let r: HashSet<_> = res
+            .drain(..)
+            .chain(
+                self.block_worlds
+                    .iter()
+                    .map(|world| rule.apply(&world.facts, &self.symbols))
+                    .flatten(),
+            )
+            .collect();
+
+        r.into_iter()
+            .map(|f| Fact::convert_from(&f, &self.symbols))
+            .map(|fact| fact.try_into().map_err(Into::into))
+            .collect()
+    }
+
     /// add a check to the authorizer
     pub fn add_check<C: TryInto<Check>>(&mut self, check: C) -> Result<(), error::Token>
     where
@@ -419,10 +499,12 @@ impl<'t> Authorizer<'t> {
 
         if let Some(token) = self.token.as_ref() {
             for (i, block) in token.blocks.iter().enumerate() {
+                let mut world = self.world.clone();
+
                 // blocks cannot provide authority or ambient facts
                 for fact in block.facts.iter().cloned() {
                     let fact = Fact::convert_from(&fact, &token.symbols).convert(&mut self.symbols);
-                    self.world.facts.insert(fact);
+                    world.facts.insert(fact);
                 }
 
                 for rule in block.rules.iter().cloned() {
@@ -437,13 +519,13 @@ impl<'t> Authorizer<'t> {
                     }
 
                     let rule = r.convert(&mut self.symbols);
-                    self.world.rules.push(rule);
+                    world.rules.push(rule);
                 }
 
-                self.world
+                world
                     .run_with_limits(&self.symbols, RunLimits::default())
                     .map_err(error::Token::RunLimit)?;
-                self.world.rules.clear();
+                world.rules.clear();
 
                 for (j, check) in block.checks.iter().enumerate() {
                     let mut successful = false;
@@ -451,7 +533,7 @@ impl<'t> Authorizer<'t> {
                     let check = c.convert(&mut self.symbols);
 
                     for query in check.queries.iter() {
-                        let res = self.world.query_match(query.clone(), &self.symbols);
+                        let res = world.query_match(query.clone(), &self.symbols);
 
                         let now = Instant::now();
                         if now >= time_limit {
@@ -472,6 +554,8 @@ impl<'t> Authorizer<'t> {
                         }));
                     }
                 }
+
+                self.block_worlds.push(world);
             }
         }
 
@@ -550,6 +634,12 @@ impl<'t> Authorizer<'t> {
             self.world
                 .facts
                 .iter()
+                .chain(
+                    self.block_worlds
+                        .iter()
+                        .map(|world| world.facts.iter())
+                        .flatten(),
+                )
                 .map(|f| Fact::convert_from(f, &self.symbols))
                 .collect(),
             self.world
@@ -618,4 +708,43 @@ mod tests {
         authorizer.add_policy("allow if true").unwrap();
         assert_eq!(authorizer.authorize(), Ok(0));
     }
+
+    #[test]
+    fn query_authorizer_from_token_tuple() {
+        use crate::Biscuit;
+        use crate::KeyPair;
+        let keypair = KeyPair::new();
+        let mut builder = Biscuit::builder(&keypair);
+        builder
+            .add_authority_fact("user(\"John Doe\", 42)")
+            .unwrap();
+
+        let biscuit = builder.build().unwrap();
+
+        let mut authorizer = biscuit.authorizer().unwrap();
+        let res: Vec<(String, i64)> = authorizer
+            .query("data($name, $id) <- user($name, $id)")
+            .unwrap();
+
+        assert_eq!(res.len(), 1);
+        assert_eq!(res[0].0, "John Doe");
+        assert_eq!(res[0].1, 42);
+    }
+
+    #[test]
+    fn query_authorizer_from_token_string() {
+        use crate::Biscuit;
+        use crate::KeyPair;
+        let keypair = KeyPair::new();
+        let mut builder = Biscuit::builder(&keypair);
+        builder.add_authority_fact("user(\"John Doe\")").unwrap();
+
+        let biscuit = builder.build().unwrap();
+
+        let mut authorizer = biscuit.authorizer().unwrap();
+        let res: Vec<(String,)> = authorizer.query("data($name) <- user($name)").unwrap();
+
+        assert_eq!(res.len(), 1);
+        assert_eq!(res[0].0, "John Doe");
+    }
 }
