@@ -6,7 +6,6 @@ use super::format::SerializedBiscuit;
 use builder::{BiscuitBuilder, BlockBuilder};
 use prost::Message;
 use rand_core::{CryptoRng, RngCore};
-use std::collections::HashSet;
 
 use crate::format::{convert::proto_block_to_token_block, schema};
 use authorizer::Authorizer;
@@ -15,22 +14,15 @@ pub mod authorizer;
 pub mod builder;
 pub mod unverified;
 
+/// minimum supported version of the serialization format
+pub const MIN_SCHEMA_VERSION: u32 = 3;
 /// maximum supported version of the serialization format
-pub const MAX_SCHEMA_VERSION: u32 = 2;
+pub const MAX_SCHEMA_VERSION: u32 = 3;
 
 /// some symbols are predefined and available in every implementation, to avoid
 /// transmitting them with every token
 pub fn default_symbol_table() -> SymbolTable {
-    let mut syms = SymbolTable::new();
-    syms.insert("authority");
-    syms.insert("ambient");
-    syms.insert("resource");
-    syms.insert("operation");
-    syms.insert("right");
-    syms.insert("current_time");
-    syms.insert("revocation_id");
-
-    syms
+    SymbolTable::new()
 }
 
 /// This structure represents a valid Biscuit token
@@ -205,7 +197,7 @@ impl Biscuit {
 
         format!(
             "Biscuit {{\n    symbols: {:?}\n    authority: {}\n    blocks: [\n        {}\n    ]\n}}",
-            self.symbols.symbols,
+            self.symbols.strings(),
             authority,
             blocks.join(",\n\t")
         )
@@ -226,7 +218,7 @@ impl Biscuit {
     }
 
     /// create the first block's builder, sing a provided symbol table
-    pub fn builder_with_symbols(root: &KeyPair, symbols: SymbolTable) -> BiscuitBuilder {
+    fn builder_with_symbols(root: &KeyPair, symbols: SymbolTable) -> BiscuitBuilder {
         BiscuitBuilder::new(root, symbols)
     }
 
@@ -240,16 +232,11 @@ impl Biscuit {
         mut symbols: SymbolTable,
         authority: Block,
     ) -> Result<Biscuit, error::Token> {
-        let h1 = symbols.symbols.iter().collect::<HashSet<_>>();
-        let h2 = authority.symbols.symbols.iter().collect::<HashSet<_>>();
-
-        if !h1.is_disjoint(&h2) {
+        if !symbols.is_disjoint(&authority.symbols) {
             return Err(error::Token::SymbolTableOverlap);
         }
 
-        symbols
-            .symbols
-            .extend(authority.symbols.symbols.iter().cloned());
+        symbols.extend(&authority.symbols);
 
         let blocks = vec![];
 
@@ -266,11 +253,7 @@ impl Biscuit {
     }
 
     /// deserializes a token and validates the signature using the root public key, with a custom symbol table
-    pub fn from_with_symbols<F>(
-        slice: &[u8],
-        f: F,
-        symbols: SymbolTable,
-    ) -> Result<Self, error::Token>
+    fn from_with_symbols<F>(slice: &[u8], f: F, symbols: SymbolTable) -> Result<Self, error::Token>
     where
         F: Fn(Option<u32>) -> PublicKey,
     {
@@ -307,14 +290,10 @@ impl Biscuit {
             blocks.push(deser);
         }
 
-        symbols
-            .symbols
-            .extend(authority.symbols.symbols.iter().cloned());
+        symbols.extend(&authority.symbols);
 
         for block in blocks.iter() {
-            symbols
-                .symbols
-                .extend(block.symbols.symbols.iter().cloned());
+            symbols.extend(&block.symbols);
         }
 
         let root_key_id = container.root_key_id;
@@ -330,7 +309,7 @@ impl Biscuit {
     }
 
     /// deserializes a token and validates the signature using the root public key, with a custom symbol table
-    pub fn from_base64_with_symbols<T, F>(
+    fn from_base64_with_symbols<T, F>(
         slice: T,
         f: F,
         symbols: SymbolTable,
@@ -363,10 +342,7 @@ impl Biscuit {
 
         let block = block_builder.build(self.symbols.clone());
 
-        let h1 = self.symbols.symbols.iter().collect::<HashSet<_>>();
-        let h2 = block.symbols.symbols.iter().collect::<HashSet<_>>();
-
-        if !h1.is_disjoint(&h2) {
+        if !self.symbols.is_disjoint(&block.symbols) {
             return Err(error::Token::SymbolTableOverlap);
         }
 
@@ -379,9 +355,7 @@ impl Biscuit {
             Some(c) => c.append(keypair, &block)?,
         };
 
-        symbols
-            .symbols
-            .extend(block.symbols.symbols.iter().cloned());
+        symbols.extend(&block.symbols);
         blocks.push(block);
 
         Ok(Biscuit {
@@ -404,7 +378,7 @@ impl Biscuit {
             }
         };
 
-        Some(block.symbols.symbols.clone())
+        Some(block.symbols.strings())
     }
 
     /// returns the number of blocks (at least 1)
@@ -449,7 +423,7 @@ fn print_block(symbols: &SymbolTable, block: &Block) -> String {
 
     format!(
         "Block {{\n            symbols: {:?}\n            version: {}\n            context: \"{}\"\n            facts: [{}]\n            rules: [{}]\n            checks: [{}]\n        }}",
-        block.symbols.symbols,
+        block.symbols.strings(),
         block.version,
         block.context.as_deref().unwrap_or(""),
         facts,
@@ -477,20 +451,6 @@ pub struct Block {
 }
 
 impl Block {
-    /// creates a new block
-    ///
-    /// blocks should be created through the BlockBuilder interface instead, to avoid mistakes
-    pub fn new(base_symbols: SymbolTable) -> Block {
-        Block {
-            symbols: base_symbols,
-            facts: vec![],
-            rules: vec![],
-            checks: vec![],
-            context: None,
-            version: MAX_SCHEMA_VERSION,
-        }
-    }
-
     pub fn symbol_add(&mut self, s: &str) -> Term {
         self.symbols.add(s)
     }
@@ -523,7 +483,7 @@ impl Block {
 
 #[cfg(test)]
 mod tests {
-    use super::builder::{check, fact, int, pred, rule, s, var};
+    use super::builder::{check, fact, pred, rule, s, var};
     use super::*;
     use crate::crypto::KeyPair;
     use crate::error::*;
@@ -966,7 +926,7 @@ mod tests {
             println!("authorization result: {:?}", authorization_res);
 
             let res2: Result<Vec<builder::Fact>, crate::error::Token> =
-                authorizer.query("key_verif($id) <- key($id)");
+                authorizer.query_all("key_verif($id) <- key($id)");
             println!("res2: {:?}", res2);
             let mut res2 = res2
                 .unwrap()
